@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2000 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2001 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -41,18 +41,18 @@
 
 #else
 
-#define pth_debug1(a1)                     pth_debug(__FILE__, __LINE__, a1)
-#define pth_debug2(a1, a2)                 pth_debug(__FILE__, __LINE__, a1, a2)
-#define pth_debug3(a1, a2, a3)             pth_debug(__FILE__, __LINE__, a1, a2, a3)
-#define pth_debug4(a1, a2, a3, a4)         pth_debug(__FILE__, __LINE__, a1, a2, a3, a4)
-#define pth_debug5(a1, a2, a3, a4, a5)     pth_debug(__FILE__, __LINE__, a1, a2, a3, a4, a5)
-#define pth_debug6(a1, a2, a3, a4, a5, a6) pth_debug(__FILE__, __LINE__, a1, a2, a3, a4, a5, a6)
+#define pth_debug1(a1)                     pth_debug(__FILE__, __LINE__, 1, a1)
+#define pth_debug2(a1, a2)                 pth_debug(__FILE__, __LINE__, 2, a1, a2)
+#define pth_debug3(a1, a2, a3)             pth_debug(__FILE__, __LINE__, 3, a1, a2, a3)
+#define pth_debug4(a1, a2, a3, a4)         pth_debug(__FILE__, __LINE__, 4, a1, a2, a3, a4)
+#define pth_debug5(a1, a2, a3, a4, a5)     pth_debug(__FILE__, __LINE__, 5, a1, a2, a3, a4, a5)
+#define pth_debug6(a1, a2, a3, a4, a5, a6) pth_debug(__FILE__, __LINE__, 6, a1, a2, a3, a4, a5, a6)
 
 #endif /* PTH_DEBUG */
 
 #endif /* cpp */
 
-intern void pth_debug(const char *file, int line, const char *fmt, ...)
+intern void pth_debug(const char *file, int line, int argc, const char *fmt, ...)
 {
     va_list ap;
     static char str[1024];
@@ -61,11 +61,14 @@ intern void pth_debug(const char *file, int line, const char *fmt, ...)
     errno_shield {
         va_start(ap, fmt);
         if (file != NULL)
-            sprintf(str, "%d:%s:%04d: ", (int)getpid(), file, line);
+            pth_snprintf(str, sizeof(str), "%d:%s:%04d: ", (int)getpid(), file, line);
         else
             str[0] = NUL;
         n = strlen(str);
-        vsprintf(str+n, fmt, ap);
+        if (argc == 1)
+            pth_util_cpystrn(str+n, fmt, sizeof(str)-n);
+        else
+            pth_vsnprintf(str+n, sizeof(str)-n, fmt, ap);
         va_end(ap);
         n = strlen(str);
         str[n++] = '\n';
