@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2000 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2001 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -43,6 +43,10 @@ int pth_usleep(unsigned int usec)
     pth_event_t ev;
     static pth_key_t ev_key = PTH_KEY_INIT;
 
+    /* short-circuit */
+    if (usec == 0)
+        return 0;
+
     /* calculate asleep time */
     offset = pth_time((long)(usec / 1000000), (long)(usec % 1000000));
     pth_time_set(&until, PTH_TIME_NOW);
@@ -63,6 +67,10 @@ unsigned int pth_sleep(unsigned int sec)
     pth_event_t ev;
     static pth_key_t ev_key = PTH_KEY_INIT;
 
+    /* consistency check */
+    if (sec == 0)
+        return 0;
+
     /* calculate asleep time */
     offset = pth_time(sec, 0);
     pth_time_set(&until, PTH_TIME_NOW);
@@ -133,6 +141,98 @@ int pth_sigwait_ev(const sigset_t *set, int *sigp, pth_event_t ev_extra)
     return 0;
 }
 
+/* Pth variant of waitpid(2) */
+pid_t pth_waitpid(pid_t wpid, int *status, int options)
+{
+    pth_event_t ev;
+    static pth_key_t ev_key = PTH_KEY_INIT;
+    pid_t pid;
+
+    pth_debug2("pth_waitpid: called from thread \"%s\"", pth_current->name);
+
+    for (;;) {
+        /* do a non-blocking poll for the pid */
+        while (   (pid = pth_sc(waitpid)(wpid, status, options|WNOHANG)) < 0 
+               && errno == EINTR) ;
+
+        /* if pid was found or caller requested a polling return immediately */
+        if (pid == -1 || pid > 0 || (pid == 0 && (options & WNOHANG)))
+            break;
+
+        /* else wait a little bit */
+        ev = pth_event(PTH_EVENT_TIME|PTH_MODE_STATIC, &ev_key, pth_timeout(0,250000));
+        pth_wait(ev);
+    }
+
+    pth_debug2("pth_waitpid: leave to thread \"%s\"", pth_current->name);
+    return pid;
+}
+
+/* Pth variant of system(3) */
+int pth_system(const char *cmd)
+{
+    struct sigaction sa_ign, sa_int, sa_quit;
+    sigset_t ss_block, ss_old;
+    struct stat sb;
+    pid_t pid;
+    int pstat;
+
+    /* POSIX calling convention: determine whether the
+       Bourne Shell ("sh") is available on this platform */
+    if (cmd == NULL) {
+        if (stat(PTH_PATH_BINSH, &sb) == -1)
+            return 0;
+        return 1;
+    }
+
+    /* temporarily ignore SIGINT and SIGQUIT actions */
+    sa_ign.sa_handler = SIG_IGN;
+    sigemptyset(&sa_ign.sa_mask);
+    sa_ign.sa_flags = 0;
+    sigaction(SIGINT,  &sa_ign, &sa_int);
+    sigaction(SIGQUIT, &sa_ign, &sa_quit);
+
+    /* block SIGCHLD signal */
+    sigemptyset(&ss_block);
+    sigaddset(&ss_block, SIGCHLD);
+    sigprocmask(SIG_BLOCK, &ss_block, &ss_old);
+
+    /* fork the current process */
+    pstat = -1;
+    switch (pid = pth_fork()) {
+        case -1: /* error */
+            break;
+
+        case 0:  /* child */
+            /* restore original signal dispositions and execute the command */
+            sigaction(SIGINT,  &sa_int,  NULL);
+            sigaction(SIGQUIT, &sa_quit, NULL);
+            sigprocmask(SIG_SETMASK, &ss_old, NULL);
+
+            /* stop the Pth scheduling */
+            pth_scheduler_kill();
+
+            /* execute the command through Bourne Shell */
+            execl(PTH_PATH_BINSH, "sh", "-c", cmd, NULL);
+
+            /* POSIX compliant return in case execution failed */
+            exit(127);
+
+        default: /* parent */
+            /* wait until child process terminates */
+            pid = pth_waitpid(pid, &pstat, 0);
+            break;
+    }
+
+    /* restore original signal dispositions and execute the command */
+    sigaction(SIGINT,  &sa_int,  NULL);
+    sigaction(SIGQUIT, &sa_quit, NULL);
+    sigprocmask(SIG_SETMASK, &ss_old, NULL);
+
+    /* return error or child process result code */
+    return (pid == -1 ? -1 : pstat);
+}
+
 /* Pth variant of select(2) */
 int pth_select(int nfds, fd_set *readfds, fd_set *writefds,
                fd_set *exceptfds, struct timeval *timeout)
@@ -629,33 +729,6 @@ ssize_t pth_write_ev(int fd, const void *buf, size_t nbytes, pth_event_t ev_extr
     return rv;
 }
 
-/* Pth variant of waitpid(2) */
-pid_t pth_waitpid(pid_t wpid, int *status, int options)
-{
-    pth_event_t ev;
-    static pth_key_t ev_key = PTH_KEY_INIT;
-    pid_t pid;
-
-    pth_debug2("pth_waitpid: called from thread \"%s\"", pth_current->name);
-
-    for (;;) {
-        /* do a non-blocking poll for the pid */
-        while (   (pid = pth_sc(waitpid)(wpid, status, options|WNOHANG)) < 0 
-               && errno == EINTR) ;
-
-        /* if pid was found or caller requested a polling return immediately */
-        if (pid == -1 || pid > 0 || (pid == 0 && (options & WNOHANG)))
-            break;
-
-        /* else wait a little bit */
-        ev = pth_event(PTH_EVENT_TIME|PTH_MODE_STATIC, &ev_key, pth_timeout(0,250000));
-        pth_wait(ev);
-    }
-
-    pth_debug2("pth_waitpid: leave to thread \"%s\"", pth_current->name);
-    return pid;
-}
-
 /* Pth variant of readv(2) */
 ssize_t pth_readv(int fd, const struct iovec *iov, int iovcnt)
 {
@@ -1044,3 +1117,185 @@ ssize_t pth_pwrite(int fd, const void *buf, size_t nbytes, off_t offset)
     return rc;
 }
 
+/* Pth variant of SUSv2 recv(2) */
+ssize_t pth_recv(int s, void *buf, size_t len, int flags)
+{
+    return pth_recv_ev(s, buf, len, flags, NULL);
+}
+
+/* Pth variant of SUSv2 recv(2) with extra event(s) */
+ssize_t pth_recv_ev(int s, void *buf, size_t len, int flags, pth_event_t ev)
+{
+    return pth_recvfrom_ev(s, buf, len, flags, NULL, 0, ev);
+}
+
+/* Pth variant of SUSv2 recvfrom(2) */
+ssize_t pth_recvfrom(int s, void *buf, size_t len, int flags, struct sockaddr *from, socklen_t *fromlen)
+{
+    return pth_recvfrom_ev(s, buf, len, flags, from, fromlen, NULL);
+}
+
+/* Pth variant of SUSv2 recvfrom(2) with extra event(s) */
+ssize_t pth_recvfrom_ev(int fd, void *buf, size_t nbytes, int flags, struct sockaddr *from, socklen_t *fromlen, pth_event_t ev_extra)
+{
+    struct timeval delay;
+    pth_event_t ev;
+    static pth_key_t ev_key = PTH_KEY_INIT;
+    fd_set fds;
+    int n;
+
+    pth_implicit_init();
+    pth_debug2("pth_recvfrom_ev: enter from thread \"%s\"", pth_current->name);
+
+    /* POSIX compliance */
+    if (nbytes == 0)
+        return 0;
+
+    /* poll filedescriptor when not already in non-blocking operation */
+    if (pth_fdmode(fd, PTH_FDMODE_POLL) == PTH_FDMODE_BLOCK) {
+
+        /* now directly poll filedescriptor for readability
+           to avoid unneccessary (and resource consuming because of context
+           switches, etc) event handling through the scheduler */
+        FD_ZERO(&fds);
+        FD_SET(fd, &fds);
+        delay.tv_sec  = 0;
+        delay.tv_usec = 0;
+        while ((n = pth_sc(select)(fd+1, &fds, NULL, NULL, &delay)) < 0
+               && errno == EINTR) ;
+
+        /* when filedescriptor is still not readable,
+           let thread sleep until it is or the extra event occurs */
+        if (n < 1) {
+            ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_READABLE|PTH_MODE_STATIC, &ev_key, fd);
+            if (ev_extra != NULL)
+                pth_event_concat(ev, ev_extra, NULL);
+            n = pth_wait(ev);
+            if (ev_extra != NULL) {
+                pth_event_isolate(ev);
+                if (!pth_event_occurred(ev))
+                    return_errno(-1, EINTR);
+            }
+        }
+    }
+
+    /* Now perform the actual read. We're now guarrantied to not block,
+       either because we were already in non-blocking mode or we determined
+       above by polling that the next recvfrom(2) call will not block.  But keep
+       in mind, that only 1 next recvfrom(2) call is guarrantied to not block
+       (except for the EINTR situation). */
+    while ((n = pth_sc(recvfrom)(fd, buf, nbytes, flags, from, fromlen)) < 0
+           && errno == EINTR) ;
+
+    pth_debug2("pth_recvfrom_ev: leave to thread \"%s\"", pth_current->name);
+    return n;
+}
+
+/* Pth variant of SUSv2 send(2) */
+ssize_t pth_send(int s, const void *buf, size_t len, int flags)
+{
+    return pth_send_ev(s, buf, len, flags, NULL);
+}
+
+/* Pth variant of SUSv2 send(2) with extra event(s) */
+ssize_t pth_send_ev(int s, const void *buf, size_t len, int flags, pth_event_t ev)
+{
+    return pth_sendto_ev(s, buf, len, flags, NULL, 0, ev);
+}
+
+/* Pth variant of SUSv2 sendto(2) */
+ssize_t pth_sendto(int s, const void *buf, size_t len, int flags, const struct sockaddr *to, socklen_t tolen)
+{
+    return pth_sendto_ev(s, buf, len, flags, to, tolen, NULL);
+}
+
+/* Pth variant of SUSv2 sendto(2) with extra event(s) */
+ssize_t pth_sendto_ev(int fd, const void *buf, size_t nbytes, int flags, const struct sockaddr *to, socklen_t tolen, pth_event_t ev_extra)
+{
+    struct timeval delay;
+    pth_event_t ev;
+    static pth_key_t ev_key = PTH_KEY_INIT;
+    fd_set fds;
+    int fdmode;
+    ssize_t rv;
+    ssize_t s;
+    int n;
+
+    pth_implicit_init();
+    pth_debug2("pth_sendto_ev: enter from thread \"%s\"", pth_current->name);
+
+    /* POSIX compliance */
+    if (nbytes == 0)
+        return 0;
+
+    /* force filedescriptor into non-blocking mode */
+    fdmode = pth_fdmode(fd, PTH_FDMODE_NONBLOCK);
+
+    /* poll filedescriptor when not already in non-blocking operation */
+    if (fdmode != PTH_FDMODE_NONBLOCK) {
+
+        /* now directly poll filedescriptor for writeability
+           to avoid unneccessary (and resource consuming because of context
+           switches, etc) event handling through the scheduler */
+        FD_ZERO(&fds);
+        FD_SET(fd, &fds);
+        delay.tv_sec  = 0;
+        delay.tv_usec = 0;
+        while ((n = pth_sc(select)(fd+1, NULL, &fds, NULL, &delay)) < 0
+               && errno == EINTR) ;
+
+        rv = 0;
+        for (;;) {
+            /* when filedescriptor is still not writeable,
+               let thread sleep until it is or event occurs */
+            if (n < 1) {
+                ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_WRITEABLE|PTH_MODE_STATIC, &ev_key, fd);
+                if (ev_extra != NULL)
+                    pth_event_concat(ev, ev_extra, NULL);
+                pth_wait(ev);
+                if (ev_extra != NULL) {
+                    pth_event_isolate(ev);
+                    if (!pth_event_occurred(ev)) {
+                        pth_fdmode(fd, fdmode);
+                        return_errno(-1, EINTR);
+                    }
+                }
+            }
+
+            /* now perform the actual send operation */
+            while ((s = pth_sc(sendto)(fd, buf, nbytes, flags, to, tolen)) < 0
+                   && errno == EINTR) ;
+            if (s > 0)
+                rv += s;
+
+            /* although we're physically now in non-blocking mode,
+               iterate unless all data is written or an error occurs, because
+               we've to mimic the usual blocking I/O behaviour of write(2). */
+            if (s > 0 && s < (ssize_t)nbytes) {
+                nbytes -= s;
+                buf = (void *)((char *)buf + s);
+                n = 0;
+                continue;
+            }
+
+            /* pass error to caller, but not for partial writes (rv > 0) */
+            if (s < 0 && rv == 0)
+                rv = -1;
+
+            /* stop looping */
+            break;
+        }
+    }
+    else {
+        /* Now perform the actual send operation */
+        while ((rv = pth_sc(sendto)(fd, buf, nbytes, flags, to, tolen)) < 0
+               && errno == EINTR) ;
+    }
+
+    /* restore filedescriptor mode */
+    errno_shield { pth_fdmode(fd, fdmode); }
+
+    pth_debug2("pth_sendto_ev: leave to thread \"%s\"", pth_current->name);
+    return rv;
+}
+
