@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2000 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2001 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -218,16 +218,17 @@ pth_t pth_spawn(pth_attr_t attr, void *(*func)(void *), void *arg)
         t->prio        = pth_current->prio;
         t->joinable    = pth_current->joinable;
         t->cancelstate = pth_current->cancelstate;
-        sprintf(t->name, "%s.child@%d=0x%lx", 
-                pth_current->name, (unsigned int)time(NULL), 
-                (unsigned long)pth_current);
+        pth_snprintf(t->name, PTH_TCB_NAMELEN, "%s.child@%d=0x%lx", 
+                     pth_current->name, (unsigned int)time(NULL), 
+                     (unsigned long)pth_current);
     }
     else {
         /* defaults */
         t->prio        = PTH_PRIO_STD;
         t->joinable    = TRUE;
         t->cancelstate = PTH_CANCEL_DEFAULT;
-        sprintf(t->name, "user/%x", (unsigned int)time(NULL));
+        pth_snprintf(t->name, PTH_TCB_NAMELEN,
+                     "user/%x", (unsigned int)time(NULL));
     }
 
     /* initialize the time points and ranges */
@@ -259,7 +260,7 @@ pth_t pth_spawn(pth_attr_t attr, void *(*func)(void *), void *arg)
     t->cleanups    = NULL;
 
     /* initialize mutex stuff */
-    pth_ring_init(&(t->mutexring), NULL);
+    pth_ring_init(&t->mutexring);
 
     /* initialize the machine context of this new thread */
     if (t->stacksize > 0) { /* the "main thread" (indicated by == 0) is special! */
@@ -491,6 +492,7 @@ int pth_suspend(pth_t t)
         return_errno(FALSE, ESRCH);
     pth_pqueue_delete(q, t);
     pth_pqueue_insert(&pth_SQ, PTH_PRIO_STD, t);
+    pth_debug2("pth_suspend: suspend thread \"%s\"\n", t->name);
     return TRUE;
 }
 
@@ -513,6 +515,7 @@ int pth_resume(pth_t t)
         default:                q = NULL;
     }
     pth_pqueue_insert(q, PTH_PRIO_STD, t);
+    pth_debug2("pth_resume: resume thread \"%s\"\n", t->name);
     return TRUE;
 }
 
