@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2000 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2001 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -117,7 +117,7 @@ struct pth_mctx_st {
 #define SWITCH_DEBUG_LINE \
         "==== THREAD CONTEXT SWITCH ==========================================="
 #ifdef PTH_DEBUG
-#define  _pth_mctx_switch_debug pth_debug(NULL, 0, SWITCH_DEBUG_LINE);
+#define  _pth_mctx_switch_debug pth_debug(NULL, 0, 1, SWITCH_DEBUG_LINE);
 #else
 #define  _pth_mctx_switch_debug /*NOP*/
 #endif
@@ -168,7 +168,7 @@ intern int pth_mctx_set(
     mctx->uc.uc_stack.ss_flags = 0;
 
     /* configure startup function (with no arguments) */
-    makecontext(&(mctx->uc), func, 1);
+    makecontext(&(mctx->uc), func, 0+1);
 
     return TRUE;
 }
@@ -450,9 +450,9 @@ static void pth_mctx_set_bootstrap(void)
  * Oh hell, I really love it when Linux guys talk about their "POSIX
  * compliant system". It's far away from POSIX compliant, IMHO. Autoconf
  * finds sigstack/sigaltstack() on Linux, yes. But it doesn't work. Why?
- * Because on Linux (for glibc < 2.1) these two functions are nothing
- * more than silly stub functions which always return just -1. Very
- * useful, yeah...
+ * Because on Linux below version 2.2 and glibc versions below 2.1 these
+ * two functions are nothing more than silly stub functions which always
+ * return just -1. Very useful, yeah...
  */
 
 #include <features.h>
@@ -497,7 +497,7 @@ pth_mctx_set(pth_mctx_t *mctx, void (*func)(void),
 {
     pth_mctx_save(mctx);
 #if i386
-    mctx->jb[4] = (int)sk_addr_hi;
+    mctx->jb[4] = (int)sk_addr_hi - sizeof(mctx->jb);
     mctx->jb[5] = (int)func;
 #else
 #error "Unsupported ISC architecture"
