@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2000 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2001 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -57,7 +57,7 @@ pth_msgport_t pth_msgport_create(const char *name)
     /* initialize structure */
     mp->mp_name  = name;
     mp->mp_tid   = pth_current;
-    pth_ring_init(&mp->mp_queue, NULL);
+    pth_ring_init(&mp->mp_queue);
 
     /* insert into list of existing message ports */
     pth_ring_append(&pth_msgport, &mp->mp_node);
@@ -79,7 +79,7 @@ void pth_msgport_destroy(pth_msgport_t mp)
         pth_msgport_reply(m);
 
     /* remove from list of existing message ports */
-    pth_ring_remove(&pth_msgport, &mp->mp_node);
+    pth_ring_delete(&pth_msgport, &mp->mp_node);
 
     /* deallocate message port structure */
     free(mp);
