@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2000 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2001 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -509,7 +509,7 @@ intern void pth_sched_eventmanager(pth_time_t *now, int dopoll)
                 }
                 /* Message Port Arrivals */
                 else if (ev->ev_type == PTH_EVENT_MSG) {
-                    if (!pth_ring_empty(&(ev->ev_args.MSG.mp->mp_queue)))
+                    if (pth_ring_elements(&(ev->ev_args.MSG.mp->mp_queue)) > 0)
                         this_occurred = TRUE;
                 }
                 /* Mutex Release */
@@ -763,8 +763,10 @@ intern void pth_sched_eventmanager(pth_time_t *now, int dopoll)
     }
 
     /* perhaps we have to internally loop... */
-    if (loop_repeat)
+    if (loop_repeat) {
+        pth_time_set(now, PTH_TIME_NOW);
         goto loop_entry;
+    }
 
     pth_debug1("pth_sched_eventmanager: leaving");
     return;
