@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2000 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2001 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -36,7 +36,6 @@ int pth_mutex_init(pth_mutex_t *mutex)
 {
     if (mutex == NULL)
         return_errno(FALSE, EINVAL);
-    pth_ring_init(NULL, &mutex->mx_node);
     mutex->mx_state = PTH_MUTEX_INITIALIZED;
     mutex->mx_owner = NULL;
     mutex->mx_count = 0;
@@ -121,7 +120,7 @@ int pth_mutex_release(pth_mutex_t *mutex)
         mutex->mx_state &= ~(PTH_MUTEX_LOCKED);
         mutex->mx_owner = NULL;
         mutex->mx_count = 0;
-        pth_ring_remove(&(pth_current->mutexring), &(mutex->mx_node));
+        pth_ring_delete(&(pth_current->mutexring), &(mutex->mx_node));
     }
     return TRUE;
 }
