@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2000 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2001 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -107,7 +107,7 @@ int pthread_attr_setinheritsched(pthread_attr_t *attr, int inheritsched)
     return_errno(ENOSYS, ENOSYS);
 }
 
-int pthread_attr_getinheritsched(pthread_attr_t *attr, int *inheritsched)
+int pthread_attr_getinheritsched(const pthread_attr_t *attr, int *inheritsched)
 {
     if (attr == NULL || inheritsched == NULL)
         return_errno(EINVAL, EINVAL);
@@ -123,7 +123,7 @@ int pthread_attr_setschedparam(pthread_attr_t *attr, struct sched_param *schedpa
     return_errno(ENOSYS, ENOSYS);
 }
 
-int pthread_attr_getschedparam(pthread_attr_t *attr, struct sched_param *schedparam)
+int pthread_attr_getschedparam(const pthread_attr_t *attr, struct sched_param *schedparam)
 {
     if (attr == NULL || schedparam == NULL)
         return_errno(EINVAL, EINVAL);
@@ -139,7 +139,7 @@ int pthread_attr_setschedpolicy(pthread_attr_t *attr, int schedpolicy)
     return_errno(ENOSYS, ENOSYS);
 }
 
-int pthread_attr_getschedpolicy(pthread_attr_t *attr, int *schedpolicy)
+int pthread_attr_getschedpolicy(const pthread_attr_t *attr, int *schedpolicy)
 {
     if (attr == NULL || schedpolicy == NULL)
         return_errno(EINVAL, EINVAL);
@@ -155,7 +155,7 @@ int pthread_attr_setscope(pthread_attr_t *attr, int scope)
     return_errno(ENOSYS, ENOSYS);
 }
 
-int pthread_attr_getscope(pthread_attr_t *attr, int *scope)
+int pthread_attr_getscope(const pthread_attr_t *attr, int *scope)
 {
     if (attr == NULL || scope == NULL)
         return_errno(EINVAL, EINVAL);
@@ -172,7 +172,7 @@ int pthread_attr_setstacksize(pthread_attr_t *attr, size_t stacksize)
     return OK;
 }
 
-int pthread_attr_getstacksize(pthread_attr_t *attr, size_t *stacksize)
+int pthread_attr_getstacksize(const pthread_attr_t *attr, size_t *stacksize)
 {
     if (attr == NULL || stacksize == NULL)
         return_errno(EINVAL, EINVAL);
@@ -190,7 +190,7 @@ int pthread_attr_setstackaddr(pthread_attr_t *attr, void *stackaddr)
     return OK;
 }
 
-int pthread_attr_getstackaddr(pthread_attr_t *attr, void **stackaddr)
+int pthread_attr_getstackaddr(const pthread_attr_t *attr, void **stackaddr)
 {
     if (attr == NULL || stackaddr == NULL)
         return_errno(EINVAL, EINVAL);
@@ -216,7 +216,7 @@ int pthread_attr_setdetachstate(pthread_attr_t *attr, int detachstate)
     return OK;
 }
 
-int pthread_attr_getdetachstate(pthread_attr_t *attr, int *detachstate)
+int pthread_attr_getdetachstate(const pthread_attr_t *attr, int *detachstate)
 {
     int s;
 
@@ -231,6 +231,22 @@ int pthread_attr_getdetachstate(pthread_attr_t *attr, int *detachstate)
     return OK;
 }
 
+int pthread_attr_setguardsize(pthread_attr_t *attr, int stacksize)
+{
+    if (attr == NULL || stacksize < 0)
+        return_errno(EINVAL, EINVAL);
+    /* not supported */
+    return_errno(ENOSYS, ENOSYS);
+}
+
+int pthread_attr_getguardsize(const pthread_attr_t *attr, int *stacksize)
+{
+    if (attr == NULL || stacksize == NULL)
+        return_errno(EINVAL, EINVAL);
+    /* not supported */
+    return_errno(ENOSYS, ENOSYS);
+}
+
 int pthread_attr_setname_np(pthread_attr_t *attr, char *name)
 {
     if (attr == NULL || name == NULL)
@@ -240,7 +256,7 @@ int pthread_attr_setname_np(pthread_attr_t *attr, char *name)
     return OK;
 }
 
-int pthread_attr_getname_np(pthread_attr_t *attr, char **name)
+int pthread_attr_getname_np(const pthread_attr_t *attr, char **name)
 {
     if (attr == NULL || name == NULL)
         return_errno(EINVAL, EINVAL);
@@ -258,7 +274,7 @@ int pthread_attr_setprio_np(pthread_attr_t *attr, int prio)
     return OK;
 }
 
-int pthread_attr_getprio_np(pthread_attr_t *attr, int *prio)
+int pthread_attr_getprio_np(const pthread_attr_t *attr, int *prio)
 {
     if (attr == NULL || prio == NULL)
         return_errno(EINVAL, EINVAL);
@@ -367,6 +383,31 @@ int pthread_kill(pthread_t thread, int sig)
     return OK;
 }
 
+/*
+**  CONCURRENCY ROUTINES
+**  
+**  We just have to provide the interface, because SUSv2 says:
+**  "The pthread_setconcurrency() function allows an application to
+**  inform the threads implementation of its desired concurrency
+**  level, new_level. The actual level of concurrency provided by the
+**  implementation as a result of this function call is unspecified."
+*/
+
+static int pthread_concurrency = 0;
+
+int pthread_getconcurrency(void)
+{
+    return pthread_concurrency;
+}
+
+int pthread_setconcurrency(int new_level)
+{
+    if (new_level < 0)
+        return_errno(EINVAL, EINVAL);
+    pthread_concurrency = new_level;
+    return OK;
+}
+
 /*
 **  CONTEXT ROUTINES
 */
@@ -989,7 +1030,6 @@ int pthread_cond_timedwait(pthread_cond_t *cond, pthread_mutex_t *mutex,
     return OK;
 }
 
-
 /*
 **  POSIX 1003.1j
 */
@@ -1078,6 +1118,30 @@ ssize_t __pthread_writev(int fd, const struct iovec *piovec, int iocnt)
     return pth_writev(fd, piovec, iocnt);
 }
 
+ssize_t __pthread_recv(int fd, void *buf, size_t nbytes, int flags)
+{
+    pthread_initialize();
+    return pth_recv(fd, buf, nbytes, flags);
+}
+
+ssize_t __pthread_send(int fd, const void *buf, size_t nbytes, int flags)
+{
+    pthread_initialize();
+    return pth_send(fd, buf, nbytes, flags);
+}
+
+ssize_t __pthread_recvfrom(int fd, void *buf, size_t nbytes, int flags, struct sockaddr *from, socklen_t *fromlen)
+{
+    pthread_initialize();
+    return pth_recvfrom(fd, buf, nbytes, flags, from, fromlen);
+}
+
+ssize_t __pthread_sendto(int fd, const void *buf, size_t nbytes, int flags, const struct sockaddr *to, socklen_t tolen)
+{
+    pthread_initialize();
+    return pth_sendto(fd, buf, nbytes, flags, to, tolen);
+}
+
 ssize_t __pthread_pread(int fd, void *buf, size_t nbytes, off_t offset)
 {
     pthread_initialize();
